@@ -2,12 +2,22 @@
 
 """
 
+import time
 import json
 import binascii
 import math
+import datetime
 
-from . import EncodeError, DecodeError
+from ..parser import EXTENSION_MARKER
+from . import EncodeError
+from . import DecodeError
 from . import compiler
+from . import format_or
+from . import utc_time_to_datetime
+from . import utc_time_from_datetime
+from . import generalized_time_to_datetime
+from . import generalized_time_from_datetime
+from .compiler import enum_values_as_dict
 
 
 class Type(object):
@@ -15,9 +25,101 @@ class Type(object):
     def __init__(self, name, type_name):
         self.name = name
         self.type_name = type_name
-        self.optional = None
+        self.optional = False
         self.default = None
 
+    def set_size_range(self, minimum, maximum, has_extension_marker):
+        pass
+
+
+class StringType(Type):
+
+    def __init__(self, name):
+        super(StringType, self).__init__(name,
+                                         self.__class__.__name__)
+
+    def encode(self, data):
+        return data
+
+    def decode(self, data):
+        return data
+
+    def __repr__(self):
+        return '{}({})'.format(self.__class__.__name__,
+                               self.name)
+
+
+class MembersType(Type):
+
+    def __init__(self,
+                 name,
+                 members,
+                 type_name):
+        super(MembersType, self).__init__(name, type_name)
+        self.members = members
+
+    def encode(self, data):
+        values = {}
+
+        for member in self.members:
+            name = member.name
+
+            if name in data:
+                try:
+                    value = member.encode(data[name])
+                except EncodeError as e:
+                    e.location.append(member.name)
+                    raise
+            elif member.optional or member.default is not None:
+                continue
+            else:
+                raise EncodeError(
+                    "{} member '{}' not found in {}.".format(
+                        self.__class__.__name__,
+                        name,
+                        data))
+
+            values[name] = value
+
+        return values
+
+    def decode(self, data):
+        values = {}
+
+        for member in self.members:
+            name = member.name
+
+            if name in data:
+                value = member.decode(data[name])
+                values[name] = value
+            elif member.optional:
+                pass
+            elif member.default is not None:
+                values[name] = member.default
+
+        return values
+
+    def __repr__(self):
+        return '{}({}, [{}])'.format(
+            self.__class__.__name__,
+            self.name,
+            ', '.join([repr(member) for member in self.members]))
+
+
+class Boolean(Type):
+
+    def __init__(self, name):
+        super(Boolean, self).__init__(name, 'BOOLEAN')
+
+    def encode(self, data):
+        return data
+
+    def decode(self, data):
+        return data
+
+    def __repr__(self):
+        return 'Boolean({})'.format(self.name)
+
 
 class Integer(Type):
 
@@ -65,10 +167,10 @@ class Real(Type):
         return 'Real({})'.format(self.name)
 
 
-class Boolean(Type):
+class Null(Type):
 
     def __init__(self, name):
-        super(Boolean, self).__init__(name, 'BOOLEAN')
+        super(Null, self).__init__(name, 'NULL')
 
     def encode(self, data):
         return data
@@ -77,136 +179,119 @@ class Boolean(Type):
         return data
 
     def __repr__(self):
-        return 'Boolean({})'.format(self.name)
+        return 'Null({})'.format(self.name)
 
 
-class IA5String(Type):
+class BitString(Type):
+
+    def __init__(self, name, minimum, maximum):
+        super(BitString, self).__init__(name, 'BIT STRING')
+
+        if minimum is None and maximum is None:
+            self.size = None
+        elif minimum == maximum:
+            self.size = minimum
+        else:
+            self.size = None
+
+    def encode(self, data):
+        value = binascii.hexlify(data[0]).decode('ascii').upper()
+
+        if self.size is None:
+            value = {
+                "value": value,
+                "length": data[1]
+            }
+
+        return value
+
+    def decode(self, data):
+        if self.size is None:
+            return (binascii.unhexlify(data['value']), data['length'])
+        else:
+            return (binascii.unhexlify(data), self.size)
+
+    def __repr__(self):
+        return 'BitString({})'.format(self.name)
+
+
+class OctetString(Type):
 
     def __init__(self, name):
-        super(IA5String, self).__init__(name, 'IA5String')
+        super(OctetString, self).__init__(name, 'OCTET STRING')
+
+    def encode(self, data):
+        return binascii.hexlify(data).decode('ascii').upper()
+
+    def decode(self, data):
+        return binascii.unhexlify(data)
+
+    def __repr__(self):
+        return 'OctetString({})'.format(self.name)
+
+
+class ObjectIdentifier(Type):
+
+    def __init__(self, name):
+        super(ObjectIdentifier, self).__init__(name, 'OBJECT IDENTIFIER')
 
     def encode(self, data):
         return data
 
     def decode(self, data):
-        return data
+        return str(data)
 
     def __repr__(self):
-        return 'IA5String({})'.format(self.name)
+        return 'ObjectIdentifier({})'.format(self.name)
 
 
-class NumericString(Type):
+class Enumerated(Type):
 
-    def __init__(self, name):
-        super(NumericString, self).__init__(name, 'NumericString')
+    def __init__(self, name, values, numeric):
+        super(Enumerated, self).__init__(name, 'ENUMERATED')
+
+        if numeric:
+            self.values = {k: k for k in enum_values_as_dict(values)}
+        else:
+            self.values = {
+                v: v for v in enum_values_as_dict(values).values()
+            }
+
+        self.has_extension_marker = (EXTENSION_MARKER in values)
+
+    def format_values(self):
+        return format_or(sorted(list(self.values)))
 
     def encode(self, data):
-        return data
+        try:
+            value = self.values[data]
+        except KeyError:
+            raise EncodeError(
+                "Expected enumeration value {}, but got '{}'.".format(
+                    self.format_values(),
+                    data))
+
+        return value
 
     def decode(self, data):
-        return data
+        if data in self.values:
+            return self.values[data]
+        elif self.has_extension_marker:
+            return None
+        else:
+            raise DecodeError(
+                "Expected enumeration value {}, but got '{}'.".format(
+                    self.format_values(),
+                    data))
 
     def __repr__(self):
-        return 'NumericString({})'.format(self.name)
+        return 'Enumerated({})'.format(self.name)
 
 
-class Sequence(Type):
+class Sequence(MembersType):
 
     def __init__(self, name, members):
-        super(Sequence, self).__init__(name, 'SEQUENCE')
-        self.members = members
-
-    def encode(self, data):
-        values = {}
-
-        for member in self.members:
-            name = member.name
-
-            if name in data:
-                value = member.encode(data[name])
-            elif member.optional or member.default is not None:
-                continue
-            else:
-                raise EncodeError(
-                    "Sequence member '{}' not found in {}.".format(
-                        name,
-                        data))
-
-            values[name] = value
-
-        return values
-
-    def decode(self, data):
-        values = {}
-
-        for member in self.members:
-            name = member.name
-
-            if name in data:
-                value = member.decode(data[name])
-                values[name] = value
-            elif member.optional:
-                pass
-            elif member.default is not None:
-                values[name] = member.default
-
-        return values
-
-    def __repr__(self):
-        return 'Sequence({}, [{}])'.format(
-            self.name,
-            ', '.join([repr(member) for member in self.members]))
-
-
-class Set(Type):
-
-    def __init__(self, name, members):
-        super(Set, self).__init__(name, 'SET')
-        self.members = members
-
-    def encode(self, data):
-        values = {}
-
-        for member in self.members:
-            name = member.name
-
-            if name in data:
-                if member.default is None:
-                    value = member.encode(data[name])
-                elif data[name] != member.default:
-                    value = member.encode(data[name])
-            elif member.optional or member.default is not None:
-                continue
-            else:
-                raise EncodeError(
-                    "Set member '{}' not found in {}.".format(
-                        name,
-                        data))
-
-            values[name] = value
-
-        return values
-
-    def decode(self, data):
-        values = {}
-
-        for member in self.members:
-            name = member.name
-
-            if name in data:
-                value = member.decode(data[name])
-                values[name] = value
-            elif member.optional:
-                pass
-            elif member.default is not None:
-                values[name] = member.default
-
-        return values
-
-    def __repr__(self):
-        return 'Set({}, [{}])'.format(
-            self.name,
-            ', '.join([repr(member) for member in self.members]))
+        super(Sequence, self).__init__(name, members, 'SEQUENCE')
 
 
 class SequenceOf(Type):
@@ -238,6 +323,12 @@ class SequenceOf(Type):
                                            self.element_type)
 
 
+class Set(MembersType):
+
+    def __init__(self, name, members):
+        super(Set, self).__init__(name, members, 'SET')
+
+
 class SetOf(Type):
 
     def __init__(self, name, element_type):
@@ -267,211 +358,46 @@ class SetOf(Type):
                                       self.element_type)
 
 
-class BitString(Type):
-
-    def __init__(self, name, minimum, maximum):
-        super(BitString, self).__init__(name, 'BIT STRING')
-
-        if minimum is None and maximum is None:
-            self.size = None
-        elif minimum == maximum:
-            self.size = minimum
-        else:
-            self.size = None
-
-    def encode(self, data):
-        value = binascii.hexlify(data[0]).decode('ascii')
-
-        if self.size is None:
-            value = {
-                "value": value,
-                "length": data[1]
-            }
-
-        return value
-
-    def decode(self, data):
-        if self.size is None:
-            return (binascii.unhexlify(data['value']), data['length'])
-        else:
-            return (binascii.unhexlify(data), self.size)
-
-    def __repr__(self):
-        return 'BitString({})'.format(self.name)
-
-
-class OctetString(Type):
-
-    def __init__(self, name):
-        super(OctetString, self).__init__(name, 'OCTET STRING')
-
-    def encode(self, data):
-        return binascii.hexlify(data).decode('ascii')
-
-    def decode(self, data):
-        return binascii.unhexlify(data)
-
-    def __repr__(self):
-        return 'OctetString({})'.format(self.name)
-
-
-class PrintableString(Type):
-
-    def __init__(self, name):
-        super(PrintableString, self).__init__(name, 'PrintableString')
-
-    def encode(self, data):
-        return data
-
-    def decode(self, data):
-        return data
-
-    def __repr__(self):
-        return 'PrintableString({})'.format(self.name)
-
-
-class UniversalString(Type):
-
-    def __init__(self, name):
-        super(UniversalString, self).__init__(name, 'UniversalString')
-
-    def encode(self, data):
-        return data
-
-    def decode(self, data):
-        return data
-
-    def __repr__(self):
-        return 'UniversalString({})'.format(self.name)
-
-
-class VisibleString(Type):
-
-    def __init__(self, name):
-        super(VisibleString, self).__init__(name, 'VisibleString')
-
-    def encode(self, data):
-        return data
-
-    def decode(self, data):
-        return data
-
-    def __repr__(self):
-        return 'VisibleString({})'.format(self.name)
-
-
-class UTF8String(Type):
-
-    def __init__(self, name):
-        super(UTF8String, self).__init__(name, 'UTF8String')
-
-    def encode(self, data):
-        return data
-
-    def decode(self, data):
-        return data
-
-    def __repr__(self):
-        return 'UTF8String({})'.format(self.name)
-
-
-class BMPString(Type):
-
-    def __init__(self, name):
-        super(BMPString, self).__init__(name, 'BMPString')
-
-    def encode(self, data):
-        return data.decode('ascii')
-
-    def decode(self, data):
-        return data.encode('ascii')
-
-    def __repr__(self):
-        return 'BMPString({})'.format(self.name)
-
-
-class UTCTime(Type):
-
-    def __init__(self, name):
-        super(UTCTime, self).__init__(name, 'UTCTime')
-
-    def encode(self, data):
-        raise NotImplementedError()
-
-    def decode(self, data):
-        raise NotImplementedError()
-
-    def __repr__(self):
-        return 'UTCTime({})'.format(self.name)
-
-
-class GeneralizedTime(Type):
-
-    def __init__(self, name):
-        super(GeneralizedTime, self).__init__(name, 'GeneralizedTime')
-
-    def encode(self, data):
-        raise NotImplementedError()
-
-    def decode(self, data):
-        raise NotImplementedError()
-
-    def __repr__(self):
-        return 'GeneralizedTime({})'.format(self.name)
-
-
-class TeletexString(Type):
-
-    def __init__(self, name):
-        super(TeletexString, self).__init__(name, 'TeletexString')
-
-    def encode(self, data):
-        return data.decode('ascii')
-
-    def decode(self, data):
-        return data.encode('ascii')
-
-    def __repr__(self):
-        return 'TeletexString({})'.format(self.name)
-
-
-class ObjectIdentifier(Type):
-
-    def __init__(self, name):
-        super(ObjectIdentifier, self).__init__(name, 'OBJECT IDENTIFIER')
-
-    def encode(self, data):
-        raise NotImplementedError()
-
-    def decode(self, data):
-        raise NotImplementedError()
-
-    def __repr__(self):
-        return 'ObjectIdentifier({})'.format(self.name)
-
-
 class Choice(Type):
 
-    def __init__(self, name, members):
+    def __init__(self, name, members, has_extension_marker):
         super(Choice, self).__init__(name, 'CHOICE')
         self.members = members
+        self.name_to_member = {member.name: member for member in self.members}
+        self.has_extension_marker = has_extension_marker
+
+    def format_names(self):
+        return format_or(sorted([member.name for member in self.members]))
 
     def encode(self, data):
-        for member in self.members:
-            if member.name in data:
-                return {member.name: member.encode(data[member.name])}
+        try:
+            member = self.name_to_member[data[0]]
+        except KeyError:
+            raise EncodeError(
+                "Expected choice {}, but got '{}'.".format(
+                    self.format_names(),
+                    data[0]))
 
-        raise EncodeError(
-            "Expected choices are {}, but got '{}'.".format(
-                [member.name for member in self.members],
-                ''.join([name for name in data])))
+        try:
+            return {member.name: member.encode(data[1])}
+        except EncodeError as e:
+            e.location.append(member.name)
+            raise
 
     def decode(self, data):
-        for member in self.members:
-            if member.name in data:
-                return {member.name: member.decode(data[member.name])}
+        name, value = list(data.items())[0]
 
-        raise DecodeError('')
+        if name in self.name_to_member:
+            member = self.name_to_member[name]
+        elif self.has_extension_marker:
+            return (None, None)
+        else:
+            raise DecodeError(
+                "Expected choice {}, but got '{}'.".format(
+                    self.format_names(),
+                    name))
+
+        return (name, member.decode(value))
 
     def __repr__(self):
         return 'Choice({}, [{}])'.format(
@@ -479,19 +405,93 @@ class Choice(Type):
             ', '.join([repr(member) for member in self.members]))
 
 
-class Null(Type):
+class UTF8String(StringType):
+    pass
 
-    def __init__(self, name):
-        super(Null, self).__init__(name, 'NULL')
+
+class NumericString(StringType):
+    pass
+
+
+class PrintableString(StringType):
+    pass
+
+
+class IA5String(StringType):
+    pass
+
+
+class VisibleString(StringType):
+    pass
+
+
+class GeneralString(StringType):
+    pass
+
+
+class BMPString(StringType):
+    pass
+
+
+class GraphicString(StringType):
+    pass
+
+
+class UniversalString(StringType):
+    pass
+
+
+class TeletexString(StringType):
+    pass
+
+
+class ObjectDescriptor(GraphicString):
+    pass
+
+
+class UTCTime(StringType):
 
     def encode(self, data):
-        return data
+        return utc_time_from_datetime(data)
 
     def decode(self, data):
-        return data
+        return utc_time_to_datetime(data)
 
-    def __repr__(self):
-        return 'Null({})'.format(self.name)
+
+class GeneralizedTime(StringType):
+
+    def encode(self, data):
+        return generalized_time_from_datetime(data)
+
+    def decode(self, data):
+        return generalized_time_to_datetime(data)
+
+
+class Date(StringType):
+
+    def encode(self, data):
+        return str(data)
+
+    def decode(self, data):
+        return datetime.date(*time.strptime(data, '%Y-%m-%d')[:3])
+
+
+class TimeOfDay(StringType):
+
+    def encode(self, data):
+        return str(data)
+
+    def decode(self, data):
+        return datetime.time(*time.strptime(data, '%H:%M:%S')[3:6])
+
+
+class DateTime(StringType):
+
+    def encode(self, data):
+        return str(data).replace(' ', 'T')
+
+    def decode(self, data):
+        return datetime.datetime(*time.strptime(data, '%Y-%m-%dT%H:%M:%S')[:6])
 
 
 class Any(Type):
@@ -499,52 +499,54 @@ class Any(Type):
     def __init__(self, name):
         super(Any, self).__init__(name, 'ANY')
 
-    def encode(self, _, encoder):
-        raise NotImplementedError()
+    def encode(self, _data):
+        raise NotImplementedError('ANY is not yet implemented.')
 
-    def decode(self, data):
-        raise NotImplementedError()
+    def decode(self, _data):
+        raise NotImplementedError('ANY is not yet implemented.')
 
     def __repr__(self):
         return 'Any({})'.format(self.name)
 
 
-class Enumerated(Type):
+class Recursive(Type, compiler.Recursive):
 
-    def __init__(self, name, values):
-        super(Enumerated, self).__init__(name, 'ENUMERATED')
-        self.values = values
+    def __init__(self, name, type_name, module_name):
+        super(Recursive, self).__init__(name, 'RECURSIVE')
+        self.type_name = type_name
+        self.module_name = module_name
+        self._inner = None
+
+    def set_inner_type(self, inner):
+        self._inner = inner
 
     def encode(self, data):
-        for name in self.values.values():
-            if data == name:
-                return data
-
-        raise EncodeError(
-            "Enumeration value '{}' not found in {}.".format(
-                data,
-                [value for value in self.values.values()]))
+        return self._inner.encode(data)
 
     def decode(self, data):
-        if data in self.values.values():
-            return data
-
-        raise DecodeError(
-            "Enumeration value '{}' not found in {}.".format(
-                data,
-                [value for value in self.values.values()]))
+        return self._inner.decode(data)
 
     def __repr__(self):
-        return 'Enumerated({})'.format(self.name)
+        return 'Recursive({})'.format(self.type_name)
 
 
-class CompiledType(object):
+class CompiledType(compiler.CompiledType):
 
     def __init__(self, type_):
+        super(CompiledType, self).__init__()
         self._type = type_
 
-    def encode(self, data):
-        string = json.dumps(self._type.encode(data), separators=(',', ':'))
+    @property
+    def type(self):
+        return self._type
+
+    def encode(self, data, indent=None):
+        dictionary = self._type.encode(data)
+
+        if indent is None:
+            string = json.dumps(dictionary, separators=(',', ':'))
+        else:
+            string = json.dumps(dictionary, indent=indent)
 
         return string.encode('utf-8')
 
@@ -557,131 +559,119 @@ class CompiledType(object):
 
 class Compiler(compiler.Compiler):
 
-    def process(self):
-        return {
-            module_name: {
-                type_name: CompiledType(self.compile_type(
-                    type_name,
-                    type_descriptor,
-                    module_name))
-                for type_name, type_descriptor
-                in self._specification[module_name]['types'].items()
-            }
-            for module_name in self._specification
-        }
+    def process_type(self, type_name, type_descriptor, module_name):
+        compiled_type = self.compile_type(type_name,
+                                          type_descriptor,
+                                          module_name)
 
-    def compile_implicit_type(self, name, type_descriptor, module_name):
-        if type_descriptor['type'] == 'SEQUENCE':
+        return CompiledType(compiled_type)
+
+    def compile_type(self, name, type_descriptor, module_name):
+        type_name = type_descriptor['type']
+
+        if type_name == 'SEQUENCE':
             members, _ = self.compile_members(
                 type_descriptor['members'],
                 module_name)
             compiled = Sequence(name, members)
-        elif type_descriptor['type'] == 'SEQUENCE OF':
-            minimum, maximum = self.get_size_range(type_descriptor,
-                                                   module_name)
+        elif type_name == 'SEQUENCE OF':
             compiled = SequenceOf(name,
                                   self.compile_type('',
                                                     type_descriptor['element'],
                                                     module_name))
-        elif type_descriptor['type'] == 'SET':
+        elif type_name == 'SET':
             members, _ = self.compile_members(
                 type_descriptor['members'],
                 module_name)
             compiled = Set(name, members)
-        elif type_descriptor['type'] == 'SET OF':
+        elif type_name == 'SET OF':
             compiled = SetOf(name,
                              self.compile_type('',
                                                type_descriptor['element'],
                                                module_name))
-        elif type_descriptor['type'] == 'CHOICE':
-            members, _ = self.compile_members(
-                type_descriptor['members'],
-                module_name)
-            compiled = Choice(name, members)
-        elif type_descriptor['type'] == 'INTEGER':
+        elif type_name == 'CHOICE':
+            compiled = Choice(name,
+                              *self.compile_members(
+                                  type_descriptor['members'],
+                                  module_name))
+        elif type_name == 'INTEGER':
             compiled = Integer(name)
-        elif type_descriptor['type'] == 'REAL':
+        elif type_name == 'REAL':
             compiled = Real(name)
-        elif type_descriptor['type'] == 'ENUMERATED':
-            compiled = Enumerated(name, type_descriptor['values'])
-        elif type_descriptor['type'] == 'BOOLEAN':
+        elif type_name == 'ENUMERATED':
+            compiled = Enumerated(name,
+                                  type_descriptor['values'],
+                                  self._numeric_enums)
+        elif type_name == 'BOOLEAN':
             compiled = Boolean(name)
-        elif type_descriptor['type'] == 'OBJECT IDENTIFIER':
+        elif type_name == 'OBJECT IDENTIFIER':
             compiled = ObjectIdentifier(name)
-        elif type_descriptor['type'] == 'OCTET STRING':
-            minimum, maximum =self.get_size_range(type_descriptor,
-                                                  module_name)
+        elif type_name == 'OCTET STRING':
             compiled = OctetString(name)
-        elif type_descriptor['type'] == 'TeletexString':
+        elif type_name == 'TeletexString':
             compiled = TeletexString(name)
-        elif type_descriptor['type'] == 'NumericString':
+        elif type_name == 'NumericString':
             compiled = NumericString(name)
-        elif type_descriptor['type'] == 'PrintableString':
+        elif type_name == 'PrintableString':
             compiled = PrintableString(name)
-        elif type_descriptor['type'] == 'IA5String':
+        elif type_name == 'IA5String':
             compiled = IA5String(name)
-        elif type_descriptor['type'] == 'VisibleString':
+        elif type_name == 'VisibleString':
             compiled = VisibleString(name)
-        elif type_descriptor['type'] == 'UTF8String':
+        elif type_name == 'GeneralString':
+            compiled = GeneralString(name)
+        elif type_name == 'DATE':
+            compiled = Date(name)
+        elif type_name == 'TIME-OF-DAY':
+            compiled = TimeOfDay(name)
+        elif type_name == 'DATE-TIME':
+            compiled = DateTime(name)
+        elif type_name == 'UTF8String':
             compiled = UTF8String(name)
-        elif type_descriptor['type'] == 'BMPString':
+        elif type_name == 'BMPString':
             compiled = BMPString(name)
-        elif type_descriptor['type'] == 'UTCTime':
+        elif type_name == 'GraphicString':
+            compiled = GraphicString(name)
+        elif type_name == 'UTCTime':
             compiled = UTCTime(name)
-        elif type_descriptor['type'] == 'UniversalString':
+        elif type_name == 'UniversalString':
             compiled = UniversalString(name)
-        elif type_descriptor['type'] == 'GeneralizedTime':
+        elif type_name == 'GeneralizedTime':
             compiled = GeneralizedTime(name)
-        elif type_descriptor['type'] == 'BIT STRING':
-            minimum, maximum =self.get_size_range(type_descriptor,
-                                                  module_name)
+        elif type_name == 'BIT STRING':
+            minimum, maximum, _ = self.get_size_range(type_descriptor,
+                                                      module_name)
             compiled = BitString(name, minimum, maximum)
-        elif type_descriptor['type'] == 'ANY':
+        elif type_name == 'ANY':
             compiled = Any(name)
-        elif type_descriptor['type'] == 'ANY DEFINED BY':
+        elif type_name == 'ANY DEFINED BY':
             compiled = Any(name)
-        elif type_descriptor['type'] == 'NULL':
+        elif type_name == 'NULL':
             compiled = Null(name)
+        elif type_name == 'EXTERNAL':
+            members, _ = self.compile_members(
+                self.external_type_descriptor()['members'],
+                module_name)
+            compiled = Sequence(name, members)
+        elif type_name == 'ObjectDescriptor':
+            compiled = ObjectDescriptor(name)
         else:
-            compiled = self.compile_type(
-                name,
-                *self.lookup_type_descriptor(
-                    type_descriptor['type'],
-                    module_name))
+            if type_name in self.types_backtrace:
+                compiled = Recursive(name,
+                                     type_name,
+                                     module_name)
+                self.recursive_types.append(compiled)
+            else:
+                compiled = self.compile_user_type(name,
+                                                  type_name,
+                                                  module_name)
 
         return compiled
 
-    def compile_type(self, name, type_descriptor, module_name):
-        compiled = self.compile_implicit_type(name,
-                                              type_descriptor,
-                                              module_name)
 
-        return compiled
-
-    def compile_members(self, members, module_name):
-        compiled_members = []
-        extension = None
-
-        for member in members:
-            if member['name'] == '...':
-                extension = []
-                continue
-
-            if extension is not None:
-                raise NotImplementedError()
-
-            compiled_member = self.compile_type(member['name'],
-                                                member,
-                                                module_name)
-            compiled_member.optional = member['optional']
-
-            if 'default' in member:
-                compiled_member.default = member['default']
-
-            compiled_members.append(compiled_member)
-
-        return compiled_members, extension
+def compile_dict(specification, numeric_enums=False):
+    return Compiler(specification, numeric_enums).process()
 
 
-def compile_dict(specification):
-    return Compiler(specification).process()
+def decode_length(_data):
+    raise DecodeError('Decode length is not supported for this codec.')
