@@ -1,4 +1,4 @@
-RFC3281 = {'PKIXAttributeCertificate': {'extensibility-implied': False,
+EXPECTED = {'PKIXAttributeCertificate': {'extensibility-implied': False,
                               'imports': {'PKIX1Explicit88': ['Attribute',
                                                               'AlgorithmIdentifier',
                                                               'CertificateSerialNumber',
@@ -28,76 +28,59 @@ RFC3281 = {'PKIXAttributeCertificate': {'extensibility-implied': False,
                                                                     'optional': True,
                                                                     'tag': {'number': 1},
                                                                     'type': 'AttrSpec'},
-                                                                   {'default': 'TRUE',
+                                                                   {'default': True,
                                                                     'name': 'permitUnSpecified',
-                                                                    'optional': False,
                                                                     'type': 'BOOLEAN'}],
                                                        'type': 'SEQUENCE'},
                                         'ACClearAttrs': {'members': [{'name': 'acIssuer',
-                                                                      'optional': False,
                                                                       'type': 'GeneralName'},
                                                                      {'name': 'acSerial',
-                                                                      'optional': False,
                                                                       'type': 'INTEGER'},
                                                                      {'element': {'type': 'Attribute'},
                                                                       'name': 'attrs',
-                                                                      'optional': False,
-                                                                      'size': None,
-                                                                      'type': 'SEQUENCE OF'}],
+                                                                      'type': 'SEQUENCE '
+                                                                              'OF'}],
                                                          'type': 'SEQUENCE'},
                                         'AttCertIssuer': {'members': [{'name': 'v1Form',
-                                                                       'optional': False,
                                                                        'type': 'GeneralNames'},
                                                                       {'name': 'v2Form',
-                                                                       'optional': False,
                                                                        'tag': {'number': 0},
                                                                        'type': 'V2Form'}],
                                                           'type': 'CHOICE'},
                                         'AttCertValidityPeriod': {'members': [{'name': 'notBeforeTime',
-                                                                               'optional': False,
                                                                                'type': 'GeneralizedTime'},
                                                                               {'name': 'notAfterTime',
-                                                                               'optional': False,
                                                                                'type': 'GeneralizedTime'}],
                                                                   'type': 'SEQUENCE'},
-                                        'AttCertVersion': {'type': 'INTEGER'},
-                                        'AttrSpec': {'element': {'type': 'OBJECT IDENTIFIER'},
-                                                     'size': None,
+                                        'AttCertVersion': {'named-numbers': {'v2': 1},
+                                                           'type': 'INTEGER'},
+                                        'AttrSpec': {'element': {'type': 'OBJECT '
+                                                                         'IDENTIFIER'},
                                                      'type': 'SEQUENCE OF'},
                                         'AttributeCertificate': {'members': [{'name': 'acinfo',
-                                                                              'optional': False,
                                                                               'type': 'AttributeCertificateInfo'},
                                                                              {'name': 'signatureAlgorithm',
-                                                                              'optional': False,
                                                                               'type': 'AlgorithmIdentifier'},
                                                                              {'name': 'signatureValue',
-                                                                              'optional': False,
-                                                                              'size': None,
-                                                                              'type': 'BIT STRING'}],
+                                                                              'type': 'BIT '
+                                                                                      'STRING'}],
                                                                  'type': 'SEQUENCE'},
                                         'AttributeCertificateInfo': {'members': [{'name': 'version',
-                                                                                  'optional': False,
                                                                                   'type': 'AttCertVersion'},
                                                                                  {'name': 'holder',
-                                                                                  'optional': False,
                                                                                   'type': 'Holder'},
                                                                                  {'name': 'issuer',
-                                                                                  'optional': False,
                                                                                   'type': 'AttCertIssuer'},
                                                                                  {'name': 'signature',
-                                                                                  'optional': False,
                                                                                   'type': 'AlgorithmIdentifier'},
                                                                                  {'name': 'serialNumber',
-                                                                                  'optional': False,
                                                                                   'type': 'CertificateSerialNumber'},
                                                                                  {'name': 'attrCertValidityPeriod',
-                                                                                  'optional': False,
                                                                                   'type': 'AttCertValidityPeriod'},
                                                                                  {'element': {'type': 'Attribute'},
                                                                                   'name': 'attributes',
-                                                                                  'optional': False,
-                                                                                  'size': None,
-                                                                                  'type': 'SEQUENCE OF'},
+                                                                                  'type': 'SEQUENCE '
+                                                                                          'OF'},
                                                                                  {'name': 'issuerUniqueID',
                                                                                   'optional': True,
                                                                                   'type': 'UniqueIdentifier'},
@@ -105,23 +88,33 @@ RFC3281 = {'PKIXAttributeCertificate': {'extensibility-implied': False,
                                                                                   'optional': True,
                                                                                   'type': 'Extensions'}],
                                                                      'type': 'SEQUENCE'},
-                                        'ClassList': {'size': None,
+                                        'ClassList': {'named-bits': [('unmarked',
+                                                                      '0'),
+                                                                     ('unclassified',
+                                                                      '1'),
+                                                                     ('restricted',
+                                                                      '2'),
+                                                                     ('confidential',
+                                                                      '3'),
+                                                                     ('secret',
+                                                                      '4'),
+                                                                     ('topSecret',
+                                                                      '5')],
                                                       'type': 'BIT STRING'},
                                         'Clearance': {'members': [{'name': 'policyId',
-                                                                   'optional': False,
                                                                    'tag': {'number': 0},
-                                                                   'type': 'OBJECT IDENTIFIER'},
-                                                                  {'default': 'unclassified',
+                                                                   'type': 'OBJECT '
+                                                                           'IDENTIFIER'},
+                                                                  {'default': ['unclassified'],
                                                                    'name': 'classList',
-                                                                   'optional': False,
                                                                    'tag': {'number': 1},
                                                                    'type': 'ClassList'},
                                                                   {'element': {'type': 'SecurityCategory'},
                                                                    'name': 'securityCategories',
                                                                    'optional': True,
-                                                                   'size': None,
                                                                    'tag': {'number': 2},
-                                                                   'type': 'SET OF'}],
+                                                                   'type': 'SET '
+                                                                           'OF'}],
                                                       'type': 'SEQUENCE'},
                                         'Holder': {'members': [{'name': 'baseCertificateID',
                                                                 'optional': True,
@@ -141,98 +134,87 @@ RFC3281 = {'PKIXAttributeCertificate': {'extensibility-implied': False,
                                                                         'tag': {'number': 0},
                                                                         'type': 'GeneralNames'},
                                                                        {'element': {'members': [{'name': 'octets',
-                                                                                                 'optional': False,
-                                                                                                 'size': None,
-                                                                                                 'type': 'OCTET STRING'},
+                                                                                                 'type': 'OCTET '
+                                                                                                         'STRING'},
                                                                                                 {'name': 'oid',
-                                                                                                 'optional': False,
-                                                                                                 'type': 'OBJECT IDENTIFIER'},
+                                                                                                 'type': 'OBJECT '
+                                                                                                         'IDENTIFIER'},
                                                                                                 {'name': 'string',
-                                                                                                 'optional': False,
                                                                                                  'type': 'UTF8String'}],
                                                                                     'type': 'CHOICE'},
                                                                         'name': 'values',
-                                                                        'optional': False,
-                                                                        'size': None,
-                                                                        'type': 'SEQUENCE OF'}],
+                                                                        'type': 'SEQUENCE '
+                                                                                'OF'}],
                                                            'type': 'SEQUENCE'},
                                         'IssuerSerial': {'members': [{'name': 'issuer',
-                                                                      'optional': False,
                                                                       'type': 'GeneralNames'},
                                                                      {'name': 'serial',
-                                                                      'optional': False,
                                                                       'type': 'CertificateSerialNumber'},
                                                                      {'name': 'issuerUID',
                                                                       'optional': True,
                                                                       'type': 'UniqueIdentifier'}],
                                                          'type': 'SEQUENCE'},
                                         'ObjectDigestInfo': {'members': [{'name': 'digestedObjectType',
-                                                                          'optional': False,
                                                                           'type': 'ENUMERATED',
-                                                                          'values': {0: 'publicKey',
-                                                                                     1: 'publicKeyCert',
-                                                                                     2: 'otherObjectTypes'}},
+                                                                          'values': [('publicKey',
+                                                                                      0),
+                                                                                     ('publicKeyCert',
+                                                                                      1),
+                                                                                     ('otherObjectTypes',
+                                                                                      2)]},
                                                                          {'name': 'otherObjectTypeID',
                                                                           'optional': True,
-                                                                          'type': 'OBJECT IDENTIFIER'},
+                                                                          'type': 'OBJECT '
+                                                                                  'IDENTIFIER'},
                                                                          {'name': 'digestAlgorithm',
-                                                                          'optional': False,
                                                                           'type': 'AlgorithmIdentifier'},
                                                                          {'name': 'objectDigest',
-                                                                          'optional': False,
-                                                                          'size': None,
-                                                                          'type': 'BIT STRING'}],
+                                                                          'type': 'BIT '
+                                                                                  'STRING'}],
                                                              'type': 'SEQUENCE'},
                                         'ProxyInfo': {'element': {'type': 'Targets'},
-                                                      'size': None,
                                                       'type': 'SEQUENCE OF'},
                                         'RoleSyntax': {'members': [{'name': 'roleAuthority',
                                                                     'optional': True,
                                                                     'tag': {'number': 0},
                                                                     'type': 'GeneralNames'},
                                                                    {'name': 'roleName',
-                                                                    'optional': False,
                                                                     'tag': {'number': 1},
                                                                     'type': 'GeneralName'}],
                                                        'type': 'SEQUENCE'},
                                         'SecurityCategory': {'members': [{'name': 'type',
-                                                                          'optional': False,
                                                                           'tag': {'kind': 'IMPLICIT',
                                                                                   'number': 0},
-                                                                          'type': 'OBJECT IDENTIFIER'},
+                                                                          'type': 'OBJECT '
+                                                                                  'IDENTIFIER'},
                                                                          {'choices': {},
                                                                           'name': 'value',
-                                                                          'optional': False,
                                                                           'tag': {'number': 1},
-                                                                          'type': 'ANY DEFINED BY',
+                                                                          'type': 'ANY '
+                                                                                  'DEFINED '
+                                                                                  'BY',
                                                                           'value': 'type'}],
                                                              'type': 'SEQUENCE'},
                                         'SvceAuthInfo': {'members': [{'name': 'service',
-                                                                      'optional': False,
                                                                       'type': 'GeneralName'},
                                                                      {'name': 'ident',
-                                                                      'optional': False,
                                                                       'type': 'GeneralName'},
                                                                      {'name': 'authInfo',
                                                                       'optional': True,
-                                                                      'size': None,
-                                                                      'type': 'OCTET STRING'}],
+                                                                      'type': 'OCTET '
+                                                                              'STRING'}],
                                                          'type': 'SEQUENCE'},
                                         'Target': {'members': [{'name': 'targetName',
-                                                                'optional': False,
                                                                 'tag': {'number': 0},
                                                                 'type': 'GeneralName'},
                                                                {'name': 'targetGroup',
-                                                                'optional': False,
                                                                 'tag': {'number': 1},
                                                                 'type': 'GeneralName'},
                                                                {'name': 'targetCert',
-                                                                'optional': False,
                                                                 'tag': {'number': 2},
                                                                 'type': 'TargetCert'}],
                                                    'type': 'CHOICE'},
                                         'TargetCert': {'members': [{'name': 'targetCertificate',
-                                                                    'optional': False,
                                                                     'type': 'IssuerSerial'},
                                                                    {'name': 'targetName',
                                                                     'optional': True,
@@ -242,7 +224,6 @@ RFC3281 = {'PKIXAttributeCertificate': {'extensibility-implied': False,
                                                                     'type': 'ObjectDigestInfo'}],
                                                        'type': 'SEQUENCE'},
                                         'Targets': {'element': {'type': 'Target'},
-                                                    'size': None,
                                                     'type': 'SEQUENCE OF'},
                                         'V2Form': {'members': [{'name': 'issuerName',
                                                                 'optional': True,
@@ -257,24 +238,29 @@ RFC3281 = {'PKIXAttributeCertificate': {'extensibility-implied': False,
                                                                 'type': 'ObjectDigestInfo'}],
                                                    'type': 'SEQUENCE'}},
                               'values': {'id-aca': {'type': 'OBJECT IDENTIFIER',
-                                                    'value': ['id-pkix',
-                                                              10]},
-                                         'id-aca-accessIdentity': {'type': 'OBJECT IDENTIFIER',
+                                                    'value': ['id-pkix', 10]},
+                                         'id-aca-accessIdentity': {'type': 'OBJECT '
+                                                                           'IDENTIFIER',
                                                                    'value': ['id-aca',
                                                                              2]},
-                                         'id-aca-authenticationInfo': {'type': 'OBJECT IDENTIFIER',
+                                         'id-aca-authenticationInfo': {'type': 'OBJECT '
+                                                                               'IDENTIFIER',
                                                                        'value': ['id-aca',
                                                                                  1]},
-                                         'id-aca-chargingIdentity': {'type': 'OBJECT IDENTIFIER',
+                                         'id-aca-chargingIdentity': {'type': 'OBJECT '
+                                                                             'IDENTIFIER',
                                                                      'value': ['id-aca',
                                                                                3]},
-                                         'id-aca-encAttrs': {'type': 'OBJECT IDENTIFIER',
+                                         'id-aca-encAttrs': {'type': 'OBJECT '
+                                                                     'IDENTIFIER',
                                                              'value': ['id-aca',
                                                                        6]},
-                                         'id-aca-group': {'type': 'OBJECT IDENTIFIER',
+                                         'id-aca-group': {'type': 'OBJECT '
+                                                                  'IDENTIFIER',
                                                           'value': ['id-aca',
                                                                     4]},
-                                         'id-at-clearance': {'type': 'OBJECT IDENTIFIER',
+                                         'id-at-clearance': {'type': 'OBJECT '
+                                                                     'IDENTIFIER',
                                                              'value': [('joint-iso-ccitt',
                                                                         2),
                                                                        ('ds',
@@ -285,18 +271,22 @@ RFC3281 = {'PKIXAttributeCertificate': {'extensibility-implied': False,
                                                                         5),
                                                                        ('clearance',
                                                                         55)]},
-                                         'id-at-role': {'type': 'OBJECT IDENTIFIER',
-                                                        'value': ['id-at',
-                                                                  72]},
-                                         'id-ce-targetInformation': {'type': 'OBJECT IDENTIFIER',
+                                         'id-at-role': {'type': 'OBJECT '
+                                                                'IDENTIFIER',
+                                                        'value': ['id-at', 72]},
+                                         'id-ce-targetInformation': {'type': 'OBJECT '
+                                                                             'IDENTIFIER',
                                                                      'value': ['id-ce',
                                                                                55]},
-                                         'id-pe-aaControls': {'type': 'OBJECT IDENTIFIER',
+                                         'id-pe-aaControls': {'type': 'OBJECT '
+                                                                      'IDENTIFIER',
                                                               'value': ['id-pe',
                                                                         6]},
-                                         'id-pe-ac-auditIdentity': {'type': 'OBJECT IDENTIFIER',
+                                         'id-pe-ac-auditIdentity': {'type': 'OBJECT '
+                                                                            'IDENTIFIER',
                                                                     'value': ['id-pe',
                                                                               4]},
-                                         'id-pe-ac-proxying': {'type': 'OBJECT IDENTIFIER',
+                                         'id-pe-ac-proxying': {'type': 'OBJECT '
+                                                                       'IDENTIFIER',
                                                                'value': ['id-pe',
-                                                                         10]}}}}
+                                                                         10]}}}}
\ No newline at end of file
