@@ -1,4 +1,4 @@
-RFC4210 = {'PKIXCMP': {'extensibility-implied': False,
+EXPECTED = {'PKIXCMP': {'extensibility-implied': False,
              'imports': {'PKCS-10': ['CertificationRequest'],
                          'PKIX1Explicit88': ['Certificate',
                                              'CertificateList',
@@ -15,75 +15,57 @@ RFC4210 = {'PKIXCMP': {'extensibility-implied': False,
              'object-sets': {},
              'tags': 'EXPLICIT',
              'types': {'CAKeyUpdAnnContent': {'members': [{'name': 'oldWithNew',
-                                                           'optional': False,
                                                            'type': 'CMPCertificate'},
                                                           {'name': 'newWithOld',
-                                                           'optional': False,
                                                            'type': 'CMPCertificate'},
                                                           {'name': 'newWithNew',
-                                                           'optional': False,
                                                            'type': 'CMPCertificate'}],
                                               'type': 'SEQUENCE'},
                        'CMPCertificate': {'members': [{'name': 'x509v3PKCert',
-                                                       'optional': False,
                                                        'type': 'Certificate'}],
                                           'type': 'CHOICE'},
                        'CRLAnnContent': {'element': {'type': 'CertificateList'},
-                                         'size': None,
                                          'type': 'SEQUENCE OF'},
                        'CertAnnContent': {'type': 'CMPCertificate'},
                        'CertConfirmContent': {'element': {'type': 'CertStatus'},
-                                              'size': None,
                                               'type': 'SEQUENCE OF'},
                        'CertOrEncCert': {'members': [{'name': 'certificate',
-                                                      'optional': False,
                                                       'tag': {'number': 0},
                                                       'type': 'CMPCertificate'},
                                                      {'name': 'encryptedCert',
-                                                      'optional': False,
                                                       'tag': {'number': 1},
                                                       'type': 'EncryptedValue'}],
                                          'type': 'CHOICE'},
                        'CertRepMessage': {'members': [{'element': {'type': 'CMPCertificate'},
                                                        'name': 'caPubs',
                                                        'optional': True,
-                                                       'size': [(1,
-                                                                 'MAX')],
+                                                       'size': [(1, 'MAX')],
                                                        'tag': {'number': 1},
                                                        'type': 'SEQUENCE OF'},
                                                       {'element': {'type': 'CertResponse'},
                                                        'name': 'response',
-                                                       'optional': False,
-                                                       'size': None,
                                                        'type': 'SEQUENCE OF'}],
                                           'type': 'SEQUENCE'},
                        'CertResponse': {'members': [{'name': 'certReqId',
-                                                     'optional': False,
                                                      'type': 'INTEGER'},
                                                     {'name': 'status',
-                                                     'optional': False,
                                                      'type': 'PKIStatusInfo'},
                                                     {'name': 'certifiedKeyPair',
                                                      'optional': True,
                                                      'type': 'CertifiedKeyPair'},
                                                     {'name': 'rspInfo',
                                                      'optional': True,
-                                                     'size': None,
                                                      'type': 'OCTET STRING'}],
                                         'type': 'SEQUENCE'},
                        'CertStatus': {'members': [{'name': 'certHash',
-                                                   'optional': False,
-                                                   'size': None,
                                                    'type': 'OCTET STRING'},
                                                   {'name': 'certReqId',
-                                                   'optional': False,
                                                    'type': 'INTEGER'},
                                                   {'name': 'statusInfo',
                                                    'optional': True,
                                                    'type': 'PKIStatusInfo'}],
                                       'type': 'SEQUENCE'},
                        'CertifiedKeyPair': {'members': [{'name': 'certOrEncCert',
-                                                         'optional': False,
                                                          'type': 'CertOrEncCert'},
                                                         {'name': 'privateKey',
                                                          'optional': True,
@@ -98,23 +80,16 @@ RFC4210 = {'PKIXCMP': {'extensibility-implied': False,
                                                   'optional': True,
                                                   'type': 'AlgorithmIdentifier'},
                                                  {'name': 'witness',
-                                                  'optional': False,
-                                                  'size': None,
                                                   'type': 'OCTET STRING'},
                                                  {'name': 'challenge',
-                                                  'optional': False,
-                                                  'size': None,
                                                   'type': 'OCTET STRING'}],
                                      'type': 'SEQUENCE'},
                        'DHBMParameter': {'members': [{'name': 'owf',
-                                                      'optional': False,
                                                       'type': 'AlgorithmIdentifier'},
                                                      {'name': 'mac',
-                                                      'optional': False,
                                                       'type': 'AlgorithmIdentifier'}],
                                          'type': 'SEQUENCE'},
                        'ErrorMsgContent': {'members': [{'name': 'pKIStatusInfo',
-                                                        'optional': False,
                                                         'type': 'PKIStatusInfo'},
                                                        {'name': 'errorCode',
                                                         'optional': True,
@@ -124,22 +99,20 @@ RFC4210 = {'PKIXCMP': {'extensibility-implied': False,
                                                         'type': 'PKIFreeText'}],
                                            'type': 'SEQUENCE'},
                        'GenMsgContent': {'element': {'type': 'InfoTypeAndValue'},
-                                         'size': None,
                                          'type': 'SEQUENCE OF'},
                        'GenRepContent': {'element': {'type': 'InfoTypeAndValue'},
-                                         'size': None,
                                          'type': 'SEQUENCE OF'},
                        'InfoTypeAndValue': {'members': [{'name': 'infoType',
-                                                         'optional': False,
-                                                         'type': 'OBJECT IDENTIFIER'},
+                                                         'type': 'OBJECT '
+                                                                 'IDENTIFIER'},
                                                         {'choices': {},
                                                          'name': 'infoValue',
                                                          'optional': True,
-                                                         'type': 'ANY DEFINED BY',
+                                                         'type': 'ANY DEFINED '
+                                                                 'BY',
                                                          'value': 'infoType'}],
                                             'type': 'SEQUENCE'},
                        'KeyRecRepContent': {'members': [{'name': 'status',
-                                                         'optional': False,
                                                          'type': 'PKIStatusInfo'},
                                                         {'name': 'newSigCert',
                                                          'optional': True,
@@ -148,17 +121,16 @@ RFC4210 = {'PKIXCMP': {'extensibility-implied': False,
                                                         {'element': {'type': 'CMPCertificate'},
                                                          'name': 'caCerts',
                                                          'optional': True,
-                                                         'size': [(1,
-                                                                   'MAX')],
+                                                         'size': [(1, 'MAX')],
                                                          'tag': {'number': 1},
                                                          'type': 'SEQUENCE OF'},
                                                         {'element': {'type': 'CertifiedKeyPair'},
                                                          'name': 'keyPairHist',
                                                          'optional': True,
-                                                         'size': [(1,
-                                                                   'MAX')],
+                                                         'size': [(1, 'MAX')],
                                                          'tag': {'number': 2},
-                                                         'type': 'SEQUENCE OF'}],
+                                                         'type': 'SEQUENCE '
+                                                                 'OF'}],
                                             'type': 'SEQUENCE'},
                        'NestedMessageContent': {'type': 'PKIMessages'},
                        'OOBCert': {'type': 'CMPCertificate'},
@@ -171,147 +143,157 @@ RFC4210 = {'PKIXCMP': {'extensibility-implied': False,
                                                     'tag': {'number': 1},
                                                     'type': 'CertId'},
                                                    {'name': 'hashVal',
-                                                    'optional': False,
-                                                    'size': None,
                                                     'type': 'BIT STRING'}],
                                        'type': 'SEQUENCE'},
                        'PBMParameter': {'members': [{'name': 'salt',
-                                                     'optional': False,
-                                                     'size': None,
                                                      'type': 'OCTET STRING'},
                                                     {'name': 'owf',
-                                                     'optional': False,
                                                      'type': 'AlgorithmIdentifier'},
                                                     {'name': 'iterationCount',
-                                                     'optional': False,
                                                      'type': 'INTEGER'},
                                                     {'name': 'mac',
-                                                     'optional': False,
                                                      'type': 'AlgorithmIdentifier'}],
                                         'type': 'SEQUENCE'},
                        'PKIBody': {'members': [{'name': 'ir',
-                                                'optional': False,
                                                 'tag': {'number': 0},
                                                 'type': 'CertReqMessages'},
                                                {'name': 'ip',
-                                                'optional': False,
                                                 'tag': {'number': 1},
                                                 'type': 'CertRepMessage'},
                                                {'name': 'cr',
-                                                'optional': False,
                                                 'tag': {'number': 2},
                                                 'type': 'CertReqMessages'},
                                                {'name': 'cp',
-                                                'optional': False,
                                                 'tag': {'number': 3},
                                                 'type': 'CertRepMessage'},
                                                {'name': 'p10cr',
-                                                'optional': False,
                                                 'tag': {'number': 4},
                                                 'type': 'CertificationRequest'},
                                                {'name': 'popdecc',
-                                                'optional': False,
                                                 'tag': {'number': 5},
                                                 'type': 'POPODecKeyChallContent'},
                                                {'name': 'popdecr',
-                                                'optional': False,
                                                 'tag': {'number': 6},
                                                 'type': 'POPODecKeyRespContent'},
                                                {'name': 'kur',
-                                                'optional': False,
                                                 'tag': {'number': 7},
                                                 'type': 'CertReqMessages'},
                                                {'name': 'kup',
-                                                'optional': False,
                                                 'tag': {'number': 8},
                                                 'type': 'CertRepMessage'},
                                                {'name': 'krr',
-                                                'optional': False,
                                                 'tag': {'number': 9},
                                                 'type': 'CertReqMessages'},
                                                {'name': 'krp',
-                                                'optional': False,
                                                 'tag': {'number': 10},
                                                 'type': 'KeyRecRepContent'},
                                                {'name': 'rr',
-                                                'optional': False,
                                                 'tag': {'number': 11},
                                                 'type': 'RevReqContent'},
                                                {'name': 'rp',
-                                                'optional': False,
                                                 'tag': {'number': 12},
                                                 'type': 'RevRepContent'},
                                                {'name': 'ccr',
-                                                'optional': False,
                                                 'tag': {'number': 13},
                                                 'type': 'CertReqMessages'},
                                                {'name': 'ccp',
-                                                'optional': False,
                                                 'tag': {'number': 14},
                                                 'type': 'CertRepMessage'},
                                                {'name': 'ckuann',
-                                                'optional': False,
                                                 'tag': {'number': 15},
                                                 'type': 'CAKeyUpdAnnContent'},
                                                {'name': 'cann',
-                                                'optional': False,
                                                 'tag': {'number': 16},
                                                 'type': 'CertAnnContent'},
                                                {'name': 'rann',
-                                                'optional': False,
                                                 'tag': {'number': 17},
                                                 'type': 'RevAnnContent'},
                                                {'name': 'crlann',
-                                                'optional': False,
                                                 'tag': {'number': 18},
                                                 'type': 'CRLAnnContent'},
                                                {'name': 'pkiconf',
-                                                'optional': False,
                                                 'tag': {'number': 19},
                                                 'type': 'PKIConfirmContent'},
                                                {'name': 'nested',
-                                                'optional': False,
                                                 'tag': {'number': 20},
                                                 'type': 'NestedMessageContent'},
                                                {'name': 'genm',
-                                                'optional': False,
                                                 'tag': {'number': 21},
                                                 'type': 'GenMsgContent'},
                                                {'name': 'genp',
-                                                'optional': False,
                                                 'tag': {'number': 22},
                                                 'type': 'GenRepContent'},
                                                {'name': 'error',
-                                                'optional': False,
                                                 'tag': {'number': 23},
                                                 'type': 'ErrorMsgContent'},
                                                {'name': 'certConf',
-                                                'optional': False,
                                                 'tag': {'number': 24},
                                                 'type': 'CertConfirmContent'},
                                                {'name': 'pollReq',
-                                                'optional': False,
                                                 'tag': {'number': 25},
                                                 'type': 'PollReqContent'},
                                                {'name': 'pollRep',
-                                                'optional': False,
                                                 'tag': {'number': 26},
                                                 'type': 'PollRepContent'}],
                                    'type': 'CHOICE'},
                        'PKIConfirmContent': {'type': 'NULL'},
-                       'PKIFailureInfo': {'size': None,
+                       'PKIFailureInfo': {'named-bits': [('badAlg', '0'),
+                                                         ('badMessageCheck',
+                                                          '1'),
+                                                         ('badRequest', '2'),
+                                                         ('badTime', '3'),
+                                                         ('badCertId', '4'),
+                                                         ('badDataFormat', '5'),
+                                                         ('wrongAuthority',
+                                                          '6'),
+                                                         ('incorrectData', '7'),
+                                                         ('missingTimeStamp',
+                                                          '8'),
+                                                         ('badPOP', '9'),
+                                                         ('certRevoked', '10'),
+                                                         ('certConfirmed',
+                                                          '11'),
+                                                         ('wrongIntegrity',
+                                                          '12'),
+                                                         ('badRecipientNonce',
+                                                          '13'),
+                                                         ('timeNotAvailable',
+                                                          '14'),
+                                                         ('unacceptedPolicy',
+                                                          '15'),
+                                                         ('unacceptedExtension',
+                                                          '16'),
+                                                         ('addInfoNotAvailable',
+                                                          '17'),
+                                                         ('badSenderNonce',
+                                                          '18'),
+                                                         ('badCertTemplate',
+                                                          '19'),
+                                                         ('signerNotTrusted',
+                                                          '20'),
+                                                         ('transactionIdInUse',
+                                                          '21'),
+                                                         ('unsupportedVersion',
+                                                          '22'),
+                                                         ('notAuthorized',
+                                                          '23'),
+                                                         ('systemUnavail',
+                                                          '24'),
+                                                         ('systemFailure',
+                                                          '25'),
+                                                         ('duplicateCertReq',
+                                                          '26')],
                                           'type': 'BIT STRING'},
                        'PKIFreeText': {'element': {'type': 'UTF8String'},
                                        'size': [(1, 'MAX')],
                                        'type': 'SEQUENCE OF'},
                        'PKIHeader': {'members': [{'name': 'pvno',
-                                                  'optional': False,
+                                                  'named-numbers': {'cmp1999': 1,
+                                                                    'cmp2000': 2},
                                                   'type': 'INTEGER'},
                                                  {'name': 'sender',
-                                                  'optional': False,
                                                   'type': 'GeneralName'},
                                                  {'name': 'recipient',
-                                                  'optional': False,
                                                   'type': 'GeneralName'},
                                                  {'name': 'messageTime',
                                                   'optional': True,
@@ -331,17 +313,14 @@ RFC4210 = {'PKIXCMP': {'extensibility-implied': False,
                                                   'type': 'KeyIdentifier'},
                                                  {'name': 'transactionID',
                                                   'optional': True,
-                                                  'size': None,
                                                   'tag': {'number': 4},
                                                   'type': 'OCTET STRING'},
                                                  {'name': 'senderNonce',
                                                   'optional': True,
-                                                  'size': None,
                                                   'tag': {'number': 5},
                                                   'type': 'OCTET STRING'},
                                                  {'name': 'recipNonce',
                                                   'optional': True,
-                                                  'size': None,
                                                   'tag': {'number': 6},
                                                   'type': 'OCTET STRING'},
                                                  {'name': 'freeText',
@@ -356,10 +335,8 @@ RFC4210 = {'PKIXCMP': {'extensibility-implied': False,
                                                   'type': 'SEQUENCE OF'}],
                                      'type': 'SEQUENCE'},
                        'PKIMessage': {'members': [{'name': 'header',
-                                                   'optional': False,
                                                    'type': 'PKIHeader'},
                                                   {'name': 'body',
-                                                   'optional': False,
                                                    'type': 'PKIBody'},
                                                   {'name': 'protection',
                                                    'optional': True,
@@ -375,10 +352,16 @@ RFC4210 = {'PKIXCMP': {'extensibility-implied': False,
                        'PKIMessages': {'element': {'type': 'PKIMessage'},
                                        'size': [(1, 'MAX')],
                                        'type': 'SEQUENCE OF'},
-                       'PKIProtection': {'size': None, 'type': 'BIT STRING'},
-                       'PKIStatus': {'type': 'INTEGER'},
+                       'PKIProtection': {'type': 'BIT STRING'},
+                       'PKIStatus': {'named-numbers': {'accepted': 0,
+                                                       'grantedWithMods': 1,
+                                                       'keyUpdateWarning': 6,
+                                                       'rejection': 2,
+                                                       'revocationNotification': 5,
+                                                       'revocationWarning': 4,
+                                                       'waiting': 3},
+                                     'type': 'INTEGER'},
                        'PKIStatusInfo': {'members': [{'name': 'status',
-                                                      'optional': False,
                                                       'type': 'PKIStatus'},
                                                      {'name': 'statusString',
                                                       'optional': True,
@@ -388,54 +371,40 @@ RFC4210 = {'PKIXCMP': {'extensibility-implied': False,
                                                       'type': 'PKIFailureInfo'}],
                                          'type': 'SEQUENCE'},
                        'POPODecKeyChallContent': {'element': {'type': 'Challenge'},
-                                                  'size': None,
                                                   'type': 'SEQUENCE OF'},
                        'POPODecKeyRespContent': {'element': {'type': 'INTEGER'},
-                                                 'size': None,
                                                  'type': 'SEQUENCE OF'},
                        'PollRepContent': {'element': {'members': [{'name': 'certReqId',
-                                                                   'optional': False,
                                                                    'type': 'INTEGER'},
                                                                   {'name': 'checkAfter',
-                                                                   'optional': False,
                                                                    'type': 'INTEGER'},
                                                                   {'name': 'reason',
                                                                    'optional': True,
                                                                    'type': 'PKIFreeText'}],
                                                       'type': 'SEQUENCE'},
-                                          'size': None,
                                           'type': 'SEQUENCE OF'},
                        'PollReqContent': {'element': {'members': [{'name': 'certReqId',
-                                                                   'optional': False,
                                                                    'type': 'INTEGER'}],
                                                       'type': 'SEQUENCE'},
-                                          'size': None,
                                           'type': 'SEQUENCE OF'},
                        'ProtectedPart': {'members': [{'name': 'header',
-                                                      'optional': False,
                                                       'type': 'PKIHeader'},
                                                      {'name': 'body',
-                                                      'optional': False,
                                                       'type': 'PKIBody'}],
                                          'type': 'SEQUENCE'},
                        'RevAnnContent': {'members': [{'name': 'status',
-                                                      'optional': False,
                                                       'type': 'PKIStatus'},
                                                      {'name': 'certId',
-                                                      'optional': False,
                                                       'type': 'CertId'},
                                                      {'name': 'willBeRevokedAt',
-                                                      'optional': False,
                                                       'type': 'GeneralizedTime'},
                                                      {'name': 'badSinceDate',
-                                                      'optional': False,
                                                       'type': 'GeneralizedTime'},
                                                      {'name': 'crlDetails',
                                                       'optional': True,
                                                       'type': 'Extensions'}],
                                          'type': 'SEQUENCE'},
                        'RevDetails': {'members': [{'name': 'certDetails',
-                                                   'optional': False,
                                                    'type': 'CertTemplate'},
                                                   {'name': 'crlEntryDetails',
                                                    'optional': True,
@@ -443,27 +412,22 @@ RFC4210 = {'PKIXCMP': {'extensibility-implied': False,
                                       'type': 'SEQUENCE'},
                        'RevRepContent': {'members': [{'element': {'type': 'PKIStatusInfo'},
                                                       'name': 'status',
-                                                      'optional': False,
-                                                      'size': [(1,
-                                                                'MAX')],
+                                                      'size': [(1, 'MAX')],
                                                       'type': 'SEQUENCE OF'},
                                                      {'element': {'type': 'CertId'},
                                                       'name': 'revCerts',
                                                       'optional': True,
-                                                      'size': [(1,
-                                                                'MAX')],
+                                                      'size': [(1, 'MAX')],
                                                       'tag': {'number': 0},
                                                       'type': 'SEQUENCE OF'},
                                                      {'element': {'type': 'CertificateList'},
                                                       'name': 'crls',
                                                       'optional': True,
-                                                      'size': [(1,
-                                                                'MAX')],
+                                                      'size': [(1, 'MAX')],
                                                       'tag': {'number': 1},
                                                       'type': 'SEQUENCE OF'}],
                                          'type': 'SEQUENCE'},
                        'RevReqContent': {'element': {'type': 'RevDetails'},
-                                         'size': None,
                                          'type': 'SEQUENCE OF'}},
              'values': {'id-DHBasedMac': {'type': 'OBJECT IDENTIFIER',
                                           'value': [1,
@@ -480,4 +444,4 @@ RFC4210 = {'PKIXCMP': {'extensibility-implied': False,
                                                           113533,
                                                           7,
                                                           66,
-                                                          13]}}}}
+                                                          13]}}}}
\ No newline at end of file
