@@ -1,4 +1,4 @@
-RFC4211 = {'PKIXCRMF-2005': {'extensibility-implied': False,
+EXPECTED = {'PKIXCRMF-2005': {'extensibility-implied': False,
                    'imports': {'CryptographicMessageSyntax2004': ['EnvelopedData'],
                                'PKIX1Explicit88': ['Version',
                                                    'AlgorithmIdentifier',
@@ -13,23 +13,21 @@ RFC4211 = {'PKIXCRMF-2005': {'extensibility-implied': False,
                    'object-sets': {},
                    'tags': 'IMPLICIT',
                    'types': {'AttributeTypeAndValue': {'members': [{'name': 'type',
-                                                                    'optional': False,
-                                                                    'type': 'OBJECT IDENTIFIER'},
+                                                                    'type': 'OBJECT '
+                                                                            'IDENTIFIER'},
                                                                    {'choices': {},
                                                                     'name': 'value',
-                                                                    'optional': False,
-                                                                    'type': 'ANY DEFINED BY',
+                                                                    'type': 'ANY '
+                                                                            'DEFINED '
+                                                                            'BY',
                                                                     'value': 'type'}],
                                                        'type': 'SEQUENCE'},
                              'Attributes': {'element': {'type': 'Attribute'},
-                                            'size': None,
                                             'type': 'SET OF'},
                              'Authenticator': {'type': 'UTF8String'},
                              'CertId': {'members': [{'name': 'issuer',
-                                                     'optional': False,
                                                      'type': 'GeneralName'},
                                                     {'name': 'serialNumber',
-                                                     'optional': False,
                                                      'type': 'INTEGER'}],
                                         'type': 'SEQUENCE'},
                              'CertReq': {'type': 'CertRequest'},
@@ -37,7 +35,6 @@ RFC4211 = {'PKIXCRMF-2005': {'extensibility-implied': False,
                                                  'size': [(1, 'MAX')],
                                                  'type': 'SEQUENCE OF'},
                              'CertReqMsg': {'members': [{'name': 'certReq',
-                                                         'optional': False,
                                                          'type': 'CertRequest'},
                                                         {'name': 'popo',
                                                          'optional': True,
@@ -45,15 +42,13 @@ RFC4211 = {'PKIXCRMF-2005': {'extensibility-implied': False,
                                                         {'element': {'type': 'AttributeTypeAndValue'},
                                                          'name': 'regInfo',
                                                          'optional': True,
-                                                         'size': [(1,
-                                                                   'MAX')],
-                                                         'type': 'SEQUENCE OF'}],
+                                                         'size': [(1, 'MAX')],
+                                                         'type': 'SEQUENCE '
+                                                                 'OF'}],
                                             'type': 'SEQUENCE'},
                              'CertRequest': {'members': [{'name': 'certReqId',
-                                                          'optional': False,
                                                           'type': 'INTEGER'},
                                                          {'name': 'certTemplate',
-                                                          'optional': False,
                                                           'type': 'CertTemplate'},
                                                          {'name': 'controls',
                                                           'optional': True,
@@ -104,23 +99,18 @@ RFC4211 = {'PKIXCRMF-2005': {'extensibility-implied': False,
                                           'size': [(1, 'MAX')],
                                           'type': 'SEQUENCE OF'},
                              'EncKeyWithID': {'members': [{'name': 'privateKey',
-                                                           'optional': False,
                                                            'type': 'PrivateKeyInfo'},
                                                           {'members': [{'name': 'string',
-                                                                        'optional': False,
                                                                         'type': 'UTF8String'},
                                                                        {'name': 'generalName',
-                                                                        'optional': False,
                                                                         'type': 'GeneralName'}],
                                                            'name': 'identifier',
                                                            'optional': True,
                                                            'type': 'CHOICE'}],
                                               'type': 'SEQUENCE'},
                              'EncryptedKey': {'members': [{'name': 'encryptedValue',
-                                                           'optional': False,
                                                            'type': 'EncryptedValue'},
                                                           {'name': 'envelopedData',
-                                                           'optional': False,
                                                            'tag': {'number': 0},
                                                            'type': 'EnvelopedData'}],
                                               'type': 'CHOICE'},
@@ -134,25 +124,23 @@ RFC4211 = {'PKIXCRMF-2005': {'extensibility-implied': False,
                                                              'type': 'AlgorithmIdentifier'},
                                                             {'name': 'encSymmKey',
                                                              'optional': True,
-                                                             'size': None,
                                                              'tag': {'number': 2},
-                                                             'type': 'BIT STRING'},
+                                                             'type': 'BIT '
+                                                                     'STRING'},
                                                             {'name': 'keyAlg',
                                                              'optional': True,
                                                              'tag': {'number': 3},
                                                              'type': 'AlgorithmIdentifier'},
                                                             {'name': 'valueHint',
                                                              'optional': True,
-                                                             'size': None,
                                                              'tag': {'number': 4},
-                                                             'type': 'OCTET STRING'},
+                                                             'type': 'OCTET '
+                                                                     'STRING'},
                                                             {'name': 'encValue',
-                                                             'optional': False,
-                                                             'size': None,
-                                                             'type': 'BIT STRING'}],
+                                                             'type': 'BIT '
+                                                                     'STRING'}],
                                                 'type': 'SEQUENCE'},
-                             'KeyGenParameters': {'size': None,
-                                                  'type': 'OCTET STRING'},
+                             'KeyGenParameters': {'type': 'OCTET STRING'},
                              'OldCertId': {'type': 'CertId'},
                              'OptionalValidity': {'members': [{'name': 'notBefore',
                                                                'optional': True,
@@ -164,70 +152,55 @@ RFC4211 = {'PKIXCRMF-2005': {'extensibility-implied': False,
                                                                'type': 'Time'}],
                                                   'type': 'SEQUENCE'},
                              'PBMParameter': {'members': [{'name': 'salt',
-                                                           'optional': False,
-                                                           'size': None,
-                                                           'type': 'OCTET STRING'},
+                                                           'type': 'OCTET '
+                                                                   'STRING'},
                                                           {'name': 'owf',
-                                                           'optional': False,
                                                            'type': 'AlgorithmIdentifier'},
                                                           {'name': 'iterationCount',
-                                                           'optional': False,
                                                            'type': 'INTEGER'},
                                                           {'name': 'mac',
-                                                           'optional': False,
                                                            'type': 'AlgorithmIdentifier'}],
                                               'type': 'SEQUENCE'},
                              'PKIArchiveOptions': {'members': [{'name': 'encryptedPrivKey',
-                                                                'optional': False,
                                                                 'tag': {'number': 0},
                                                                 'type': 'EncryptedKey'},
                                                                {'name': 'keyGenParameters',
-                                                                'optional': False,
                                                                 'tag': {'number': 1},
                                                                 'type': 'KeyGenParameters'},
                                                                {'name': 'archiveRemGenPrivKey',
-                                                                'optional': False,
                                                                 'tag': {'number': 2},
                                                                 'type': 'BOOLEAN'}],
                                                    'type': 'CHOICE'},
                              'PKIPublicationInfo': {'members': [{'name': 'action',
-                                                                 'optional': False,
+                                                                 'named-numbers': {'dontPublish': 0,
+                                                                                   'pleasePublish': 1},
                                                                  'type': 'INTEGER'},
                                                                 {'element': {'type': 'SinglePubInfo'},
                                                                  'name': 'pubInfos',
                                                                  'optional': True,
                                                                  'size': [(1,
                                                                            'MAX')],
-                                                                 'type': 'SEQUENCE OF'}],
+                                                                 'type': 'SEQUENCE '
+                                                                         'OF'}],
                                                     'type': 'SEQUENCE'},
                              'PKMACValue': {'members': [{'name': 'algId',
-                                                         'optional': False,
                                                          'type': 'AlgorithmIdentifier'},
                                                         {'name': 'value',
-                                                         'optional': False,
-                                                         'size': None,
                                                          'type': 'BIT STRING'}],
                                             'type': 'SEQUENCE'},
                              'POPOPrivKey': {'members': [{'name': 'thisMessage',
-                                                          'optional': False,
-                                                          'size': None,
                                                           'tag': {'number': 0},
                                                           'type': 'BIT STRING'},
                                                          {'name': 'subsequentMessage',
-                                                          'optional': False,
                                                           'tag': {'number': 1},
                                                           'type': 'SubsequentMessage'},
                                                          {'name': 'dhMAC',
-                                                          'optional': False,
-                                                          'size': None,
                                                           'tag': {'number': 2},
                                                           'type': 'BIT STRING'},
                                                          {'name': 'agreeMAC',
-                                                          'optional': False,
                                                           'tag': {'number': 3},
                                                           'type': 'PKMACValue'},
                                                          {'name': 'encryptedKey',
-                                                          'optional': False,
                                                           'tag': {'number': 4},
                                                           'type': 'EnvelopedData'}],
                                              'type': 'CHOICE'},
@@ -236,37 +209,28 @@ RFC4211 = {'PKIXCRMF-2005': {'extensibility-implied': False,
                                                              'tag': {'number': 0},
                                                              'type': 'POPOSigningKeyInput'},
                                                             {'name': 'algorithmIdentifier',
-                                                             'optional': False,
                                                              'type': 'AlgorithmIdentifier'},
                                                             {'name': 'signature',
-                                                             'optional': False,
-                                                             'size': None,
-                                                             'type': 'BIT STRING'}],
+                                                             'type': 'BIT '
+                                                                     'STRING'}],
                                                 'type': 'SEQUENCE'},
                              'POPOSigningKeyInput': {'members': [{'members': [{'name': 'sender',
-                                                                               'optional': False,
                                                                                'tag': {'number': 0},
                                                                                'type': 'GeneralName'},
                                                                               {'name': 'publicKeyMAC',
-                                                                               'optional': False,
                                                                                'type': 'PKMACValue'}],
                                                                   'name': 'authInfo',
-                                                                  'optional': False,
                                                                   'type': 'CHOICE'},
                                                                  {'name': 'publicKey',
-                                                                  'optional': False,
                                                                   'type': 'SubjectPublicKeyInfo'}],
                                                      'type': 'SEQUENCE'},
                              'PrivateKeyInfo': {'members': [{'name': 'version',
-                                                             'optional': False,
                                                              'type': 'INTEGER'},
                                                             {'name': 'privateKeyAlgorithm',
-                                                             'optional': False,
                                                              'type': 'AlgorithmIdentifier'},
                                                             {'name': 'privateKey',
-                                                             'optional': False,
-                                                             'size': None,
-                                                             'type': 'OCTET STRING'},
+                                                             'type': 'OCTET '
+                                                                     'STRING'},
                                                             {'name': 'attributes',
                                                              'optional': True,
                                                              'tag': {'kind': 'IMPLICIT',
@@ -274,36 +238,38 @@ RFC4211 = {'PKIXCRMF-2005': {'extensibility-implied': False,
                                                              'type': 'Attributes'}],
                                                 'type': 'SEQUENCE'},
                              'ProofOfPossession': {'members': [{'name': 'raVerified',
-                                                                'optional': False,
                                                                 'tag': {'number': 0},
                                                                 'type': 'NULL'},
                                                                {'name': 'signature',
-                                                                'optional': False,
                                                                 'tag': {'number': 1},
                                                                 'type': 'POPOSigningKey'},
                                                                {'name': 'keyEncipherment',
-                                                                'optional': False,
                                                                 'tag': {'number': 2},
                                                                 'type': 'POPOPrivKey'},
                                                                {'name': 'keyAgreement',
-                                                                'optional': False,
                                                                 'tag': {'number': 3},
                                                                 'type': 'POPOPrivKey'}],
                                                    'type': 'CHOICE'},
                              'ProtocolEncrKey': {'type': 'SubjectPublicKeyInfo'},
                              'RegToken': {'type': 'UTF8String'},
                              'SinglePubInfo': {'members': [{'name': 'pubMethod',
-                                                            'optional': False,
+                                                            'named-numbers': {'dontCare': 0,
+                                                                              'ldap': 3,
+                                                                              'web': 2,
+                                                                              'x500': 1},
                                                             'type': 'INTEGER'},
                                                            {'name': 'pubLocation',
                                                             'optional': True,
                                                             'type': 'GeneralName'}],
                                                'type': 'SEQUENCE'},
-                             'SubsequentMessage': {'type': 'INTEGER'},
+                             'SubsequentMessage': {'named-numbers': {'challengeResp': 1,
+                                                                     'encrCert': 0},
+                                                   'type': 'INTEGER'},
                              'UTF8Pairs': {'type': 'UTF8String'}},
                    'values': {'id-ct': {'type': 'OBJECT IDENTIFIER',
                                         'value': ['id-smime', 1]},
-                              'id-ct-encKeyWithID': {'type': 'OBJECT IDENTIFIER',
+                              'id-ct-encKeyWithID': {'type': 'OBJECT '
+                                                             'IDENTIFIER',
                                                      'value': ['id-ct', 21]},
                               'id-pkip': {'type': 'OBJECT IDENTIFIER',
                                           'value': ['id-pkix', 5]},
@@ -318,30 +284,38 @@ RFC4211 = {'PKIXCRMF-2005': {'extensibility-implied': False,
                                                     7]},
                               'id-regCtrl': {'type': 'OBJECT IDENTIFIER',
                                              'value': ['id-pkip', 1]},
-                              'id-regCtrl-authenticator': {'type': 'OBJECT IDENTIFIER',
+                              'id-regCtrl-authenticator': {'type': 'OBJECT '
+                                                                   'IDENTIFIER',
                                                            'value': ['id-regCtrl',
                                                                      2]},
-                              'id-regCtrl-oldCertID': {'type': 'OBJECT IDENTIFIER',
+                              'id-regCtrl-oldCertID': {'type': 'OBJECT '
+                                                               'IDENTIFIER',
                                                        'value': ['id-regCtrl',
                                                                  5]},
-                              'id-regCtrl-pkiArchiveOptions': {'type': 'OBJECT IDENTIFIER',
+                              'id-regCtrl-pkiArchiveOptions': {'type': 'OBJECT '
+                                                                       'IDENTIFIER',
                                                                'value': ['id-regCtrl',
                                                                          4]},
-                              'id-regCtrl-pkiPublicationInfo': {'type': 'OBJECT IDENTIFIER',
+                              'id-regCtrl-pkiPublicationInfo': {'type': 'OBJECT '
+                                                                        'IDENTIFIER',
                                                                 'value': ['id-regCtrl',
                                                                           3]},
-                              'id-regCtrl-protocolEncrKey': {'type': 'OBJECT IDENTIFIER',
+                              'id-regCtrl-protocolEncrKey': {'type': 'OBJECT '
+                                                                     'IDENTIFIER',
                                                              'value': ['id-regCtrl',
                                                                        6]},
-                              'id-regCtrl-regToken': {'type': 'OBJECT IDENTIFIER',
+                              'id-regCtrl-regToken': {'type': 'OBJECT '
+                                                              'IDENTIFIER',
                                                       'value': ['id-regCtrl',
                                                                 1]},
                               'id-regInfo': {'type': 'OBJECT IDENTIFIER',
                                              'value': ['id-pkip', 2]},
-                              'id-regInfo-certReq': {'type': 'OBJECT IDENTIFIER',
+                              'id-regInfo-certReq': {'type': 'OBJECT '
+                                                             'IDENTIFIER',
                                                      'value': ['id-regInfo',
                                                                2]},
-                              'id-regInfo-utf8Pairs': {'type': 'OBJECT IDENTIFIER',
+                              'id-regInfo-utf8Pairs': {'type': 'OBJECT '
+                                                               'IDENTIFIER',
                                                        'value': ['id-regInfo',
                                                                  1]},
                               'id-smime': {'type': 'OBJECT IDENTIFIER',
@@ -351,4 +325,4 @@ RFC4211 = {'PKIXCRMF-2005': {'extensibility-implied': False,
                                                      ('rsadsi', 113549),
                                                      ('pkcs', 1),
                                                      ('pkcs9', 9),
-                                                     16]}}}}
+                                                     16]}}}}
\ No newline at end of file
